#!/bin/sh
#
# jstimeline-testrunner.sh: automated test runner for jstimeline
# 
# jstimeline, basic timeline information plugin function test
#             for GNU Flash Player
# Copyright (C) 2015 Nutchanon Wetchasit
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# 
# Usage:
#     ./jstimeline-testrunner.sh
# 
# Exit codes:
#     0         if all test(s) passed
#     non-zero  if there are failed test(s), or tester encountered an error
# 
# Note:
#     This test requires a writable test directory, and a filesystem
#     that supports named pipe.
#

# Filenames and constants
LOGFILE=output.log
PIPE2CONTAINER=pipe2container
PIPE2PLAYER=pipe2toplayer
READTIMEOUT=5

# Test counts
TESTED=0
FAILED=0
PASSED=0

# check_equals($op1, $op2, $msg)
# Equality checker and counter
check_equals() {
	if [ "$1" = "$2" ]
	then
		echo "PASSED: $3"
		PASSED=`expr "$PASSED" + 1`
	else
		echo "FAILED: $3 (\"$1\" != \"$2\")"
		FAILED=`expr "$FAILED" + 1`
	fi
	TESTED=`expr "$TESTED" + 1`
}

# check_totals($op, $msg)
# Test count checker
check_totals() {
	check_equals "$TESTED" "$1" "$2"
}

# check_error($bool, $msg)
# Assert $bool is 0; if not, flag error in the test, and exit
check_error() {
	if [ "$1" -ne 0 ]
	then
		echo "ERROR: $2" >&2
		exit 1
	fi
}

# read_timeout($varname, $timeout)
# Read one line from standard input, with a specified timeout (in seconds)
read_timeout() {
	trap 'trap - USR1; return 142' USR1
	(sleep "$2" && kill -USR1 "$$" > /dev/null 2>&1) &
	TIMEOUTPID=$!
	read "$1"
	READERROR=$?
	kill "$TIMEOUTPID" > /dev/null 2>&1
	trap - USR1
	return $READERROR
}

# Create required named pipes
if [ \! -p "$PIPE2CONTAINER" ]
then
	mkfifo "$PIPE2CONTAINER"
	check_error "$?" "Failed to create a named pipe: $PIPE2CONTAINER"
fi
if [ \! -p "$PIPE2PLAYER" ]
then
	mkfifo "$PIPE2PLAYER"
	check_error "$?" "Failed to create a named pipe: $PIPE2PLAYER"
fi

# Open player-to-host pipe
exec 3<> "$PIPE2CONTAINER"
check_error $? "Failed to open a named pipe: $PIPE2CONTAINER"

# Open host-to-player pipe
exec 4<> "$PIPE2PLAYER"
check_error $? "Failed to open a named pipe: $PIPE2PLAYER"

# Start player
"gnash" -r 0 -vv -F 3:4 "jstimeline.swf" > "$LOGFILE" 2>&1 &
GNASHPID=$!

# Wait until the SWF code start running, by loop-checking logfile
STARTCOUNTDOWN=$READTIMEOUT
while [ $STARTCOUNTDOWN -gt 0 ]
do
	if grep "TRACE: STARTOFTEST" "$LOGFILE" 2>&1 > /dev/null
	then
		break
	fi
	sleep 1
	STARTCOUNTDOWN=`expr $STARTCOUNTDOWN - 1`
done

[ $STARTCOUNTDOWN -ne 0 ]
check_equals $? 0 "Gnash-side SWF code should be successfully started"

# Call number-returning PercentLoaded() plugin function
echo '<invoke name="PercentLoaded" returntype="xml"><arguments></arguments></invoke>' >&4

# Read return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<number>100</number>' "Gnash should return a correct value from PercentLoaded call"

# Call number-returning TotalFrames() plugin function
echo '<invoke name="TotalFrames" returntype="xml"><arguments></arguments></invoke>' >&4

# Read return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<number>12</number>' "Gnash should return a correct value from TotalFrames call"

# Call boolean-returning IsPlaying() plugin function
echo '<invoke name="IsPlaying" returntype="xml"><arguments></arguments></invoke>' >&4

# Read return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<true/>' "Gnash should return a correct value from IsPlaying call"

# Close pipes
exec 3<&-
exec 4<&-

# Force Gnash to exit
kill $GNASHPID
wait $GNASHPID

# Check for total number of test run
check_totals "4" "There should be 4 tests run"

# Remove temporary files
rm "$PIPE2CONTAINER"
rm "$PIPE2PLAYER"

# Exit with code 0 if no failure, otherwise 1
if [ "$FAILED" -eq "0" ]
then
	exit 0
else
	exit 1
fi
